% DEMO on running classification on the provided BOW features
% copyright (c) 2010-2012
% Fuxin Li - fli@cc.gatech.edu
% Catalin Ionescu - catalin.ionescu@ins.uni-bonn.de
% Cristian Sminchisescu - cristian.sminchisescu@ins.uni-bonn.de

% Classification DEMO with different linear approximations
%% Chebyshev
d = 3000;
addpath('./liblinear')
% Set to the number of cores you have
setenv('OMP_NUM_THREADS','2');
setenv('MKL_NUM_THREADS','2');
load bow_features_train;
opt.Nperdim = 10;
opt.method = 'chebyshev';
kobj = InitExplicitKernel('exp_chi2',1.5, size(Feats_all,2), d,opt);
z_omega = rf_featurize(kobj, double(Feats_all));
svmmodel = svmtrain_chi2(expanded_labels_all, z_omega,'-t 0 -q -c 50');

load bow_features_test;
z_omega = rf_featurize(kobj, double(Feats_all));
disp('Chebyshev: ');
[pred, accuracy] = svmpredict_chi2(expanded_labels_all, z_omega, svmmodel);
%% PCA-Chebyshev
load bow_features_train;
kobj2 = InitExplicitKernel('exp_chi2',1.5, size(Feats_all,2), d*3,opt);

[z_omega,kobj2] = rf_pca_featurize(kobj2, double(Feats_all),true,d);
svmmodel = svmtrain_chi2(expanded_labels_all, z_omega,'-t 0 -q -c 50');
var = load('bow_features_test');
z_omega = rf_pca_featurize(kobj2, double(var.Feats_all));
disp('PCA: ');
[pred, accuracy] = svmpredict_chi2(var.expanded_labels_all, z_omega,svmmodel);
clear kobj2;
%% VZ
load bow_features_train;
opt.Nperdim = 10;
opt.method = 'signals';
kobj2 = InitExplicitKernel('exp_chi2',1.5, size(Feats_all,2), d, opt);
% Use the same omega2 for all the approximations
kobj2.omega2 = kobj.omega2;
z_omega = rf_featurize(kobj2,double(Feats_all));
svmmodel = svmtrain_chi2(expanded_labels_all,z_omega,'-t 0 -q -c 50');
load bow_features_test;
z_omega = rf_featurize(kobj2, double(Feats_all));
disp('VZ: ');
[pred_accuracy] = svmpredict_chi2(expanded_labels_all,z_omega, svmmodel);
%% Nystrom
load bow_features_train;
opt.method = 'nystrom';
s1 = randperm(size(Feats_all,1));
if d > size(Feats_all,1)
    opt.omega = Feats_all';
else
    opt.omega = Feats_all(s1(1:d),:)';
end
kobj = InitExplicitKernel('exp_chi2',1.5, size(Feats_all,2), d,opt);
z_omega = rf_featurize(kobj, double(Feats_all));
svmmodel = svmtrain_chi2(expanded_labels_all, z_omega,'-t 0 -q -c 50');

load bow_features_test;
z_omega = rf_featurize(kobj, double(Feats_all));
disp('Nystrom: ');
[pred, accuracy] = svmpredict_chi2(expanded_labels_all, z_omega, svmmodel);